@@ -1,5 +1,5 @@
 /*
-  Copyright (C) 1997,1998,1999  Dimitrios P. Bouras
+  Copyright (C) 1997-2002  Dimitrios P. Bouras
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -20,41 +20,53 @@
 
 /* Constants for xisp, xispdial, rcio.c and xisprccv.c */
 
-#define MAXLEN_DESCR    32	/* configuration description string length */
-#define MAXLEN_PHONE    64	/* phone string length */
-#define MAXNUM_TELS      8	/* maximum telephone numbers in options */
-#define MAXLEN_ACCOUNT  64	/* account number of characters */
-#define MAXLEN_PASSWD   64	/* password characters */
-#define MAXLEN_UNR      64	/* name/user/remote pppd opt arg maximum chars */ 
-#define MAXDIG_RETRY     3	/* number of retries maximum digits */
-#define MAXDIG_DELAY     3	/* inter-dialing delay seconds # maximum digits */
-#define MAXDIG_CNWAIT    3	/* connection wait seconds # of digits */
-#define MAXDIG_CBDELAY   3	/* call-back delay seconds # maximum digits */
-#define MAXNUM_SLINES   16	/* maximum number of saved script lines */
-#define MAXLEN_SLINE    64	/* characters in each script line */
-#define MAXLEN_MDMCMD   64	/* modem commands maximum length */
-#define MAXLEN_MDMSTR   16	/* modem result strings maximum length */
-#define MAXLEN_DEVICE   32	/* modem device name string length */
-#define MAXDIG_BSDCOMP   2	/* BSD compression level number digits */
-#define MAXLEN_DIALEXTRA 8	/* extra chars between "AT" and "D" commands */
-#define MAXDIG_ASYNCMAP  8	/* pppd asyncmap number of digits */
-#define MAXLEN_ESCAPE   16	/* pppd escape character list length */
-#define MAXLEN_IP       15	/* all IP input strings length */
-#define MAXDIG_MTRU      4	/* MTU and MRU maximum number of digits */
-#define MAXLEN_TTPARM   64	/* xispterm parameter string maximum length */
-#define MAXBUF_CHILD   128	/* buffer size for dialer output reads by xISP */
-#define MAXNUM_RETRY    10	/* default total number of dialing tries */
-#define MAXSEC_DELAY     5	/* default delay between dials */
-#define MAXSEC_CBDELAY  60	/* default delay for call-back wait */
-#define MAXSEC_CNWAIT   50	/* default maximum wait for modem connection */
-#define MAXLEN_PATH    256  /* maximum path length for all program paths */
-#define MAXLEN_FNAME   512	/* maximum complete file-name string length */
+#define MAXLEN_DESCR     32	/* configuration description string length */
+#define MAXLEN_PHONE     64	/* phone string length */
+#define MAXNUM_TELS       8	/* maximum telephone numbers in options */
+#define MAXLEN_ACCOUNT   64	/* account number of characters */
+#define MAXLEN_PASSWD    64	/* password characters */
+#define MAXLEN_UNR       64	/* name/user/remote pppd opt arg maximum chars */ 
+#define MAXDIG_RETRY      3	/* number of retries maximum digits */
+#define MAXDIG_DELAY      3	/* inter-dialing delay seconds # maximum digits */
+#define MAXDIG_CNWAIT     3	/* modem connection wait seconds # of digits */
+#define MAXDIG_LCPWAIT    3	/* network connection wait seconds # of digits */
+#define MAXDIG_CBDELAY    3	/* call-back delay seconds # maximum digits */
+#define MAXNUM_SLINES    16	/* maximum number of saved script lines */
+#define MAXLEN_SLINE     64	/* characters in each script line */
+#define MAXLEN_MDMCMD    64	/* modem commands maximum length */
+#define MAXLEN_MDMSTR    16	/* modem result strings maximum length */
+#define MAXLEN_DEVICE    32	/* modem device name string length */
+#define MAXDIG_BSDCOMP    2	/* BSD compression level number digits */
+#define MAXLEN_DIALEXTRA  8	/* extra chars between "AT" and "D" commands */
+#define MAXDIG_ASYNCMAP   8	/* pppd asyncmap number of digits */
+#define MAXLEN_ESCAPE    16	/* pppd escape character list length */
+#define MAXLEN_IP        15	/* all IP input strings length */
+#define MAXDIG_MTRU       4	/* MTU and MRU maximum number of digits */
+#define MAXLEN_TTPARM    64	/* xispterm parameter string maximum length */
+#define MAXBUF_CHILD    128	/* buffer size for dialer output reads by xISP */
+#define MAXNUM_RETRY     10	/* default total number of dialing tries */
+#define MAXSEC_DELAY      5	/* default delay between dials */
+#define MAXSEC_CBDELAY   60	/* default delay for call-back wait */
+#define MAXSEC_CNWAIT    50	/* default maximum wait for modem connection */
+#define MAXSEC_LCPWAIT   30	/* default maximum wait for network connection */
+#define MINSEC_MODEM_TO  10	/* minimum timeout for modem connection */
+#define MINSEC_LINK_TO   15 /* minimum timeout for link connection */
+#define MAXLEN_PATH     256 /* maximum path length for all program paths */
+#define MAXLEN_FNAME    512	/* maximum complete file-name string length */
+#define MAXLEN_DNNAME    64	/* default domain name maximum string length */
+#define MAXDIG_BAUDRATE   7 /* modem speed maximum digits */
+#define MIN_COMPLEVEL     8 /* minimum (usefull) S/W compression level */
+#define MAX_COMPLEVEL    15 /* maximum S/W compression level */
 
 /* xISP file names */
 
 #define RCFNAME      ".xisprc"		/* for all program options */
 #define PIPEFNAME    ".xisppipe"	/* named pipe for reporting */
-#define ENVFNAME     ".xispenv"		/* generated dialer environment */
+#ifdef ISPENV_USEVARS
+ #define ENVVAR      "XISPDIAL"		/* generated dialer environment var */
+#else
+ #define ENVFNAME    ".xispenv"		/* generated dialer environment file */
+#endif
 #define SCRIPTFNAME  ".xispscript"	/* generated dialing script */
 #define PAPFNAME     ".xisppap"		/* the login data file for PAP */
 #define LOGDIRNAME   ".xisplogs"	/* directory holding xisp logs */
@@ -77,6 +89,9 @@
 #else
  #define RUN_PATH	 ""
 #endif
+#ifndef LOCK_PATH
+ #define LOCK_PATH   PPPDLCKDIR					/* modem device lock file */
+#endif
 #ifndef CHAT_PATH
  #define CHAT_PATH	 BINPATH					/* chat executable */
 #endif
@@ -92,25 +107,28 @@
 #ifndef MODEM_PATH
  #define MODEM_PATH    "/dev/modem"			/* default modem device */
 #endif
-#define MS_1200        0x01					/* selectable DTE<->DCE speeds */
-#define MS_2400        0x02
-#define MS_4800        0x04
-#define MS_9600        0x08
-#define MS_19200       0x10
-#define MS_38400       0x20
-#define MS_57600       0x40
-#define MS_115200      0x80
-#define MODEM_SPEED    MS_38400				/* 38400 Baud set by default */
-#define MODEM_INIT	   "AT"					/* default init string */
-#define MODEM_RESET	   "ATZ"				/* default reset string */
-#define MODEM_CONNECT  "CONNECT"			/* default connect string */
-#define BSDCOMP        "bsdcomp"			/* pppd BSD compression option */
-#define DIAL_EXTRA     ""					/* default extra dialing chars */
-#define COMP_LEVEL     12
-#define PPPD_SASYNCMAP "20A0000"			/* pppd asyncmap for software */
-#define PPPD_HASYNCMAP "00"					/* and hardware flow contol */
-#define PPPD_ESCAPE    "FF"					/* default pppd escape list */
-#define DEFLCOMP       "deflate"			/* pppd deflate compression opt */
+#define MAXNUM_SPEEDS   8					/* DTE<->DCE speeds: a subset */
+#define MS_0            1200				/* of the "standard" ones */
+#define MS_1            2400
+#define MS_2            4800
+#define MS_3            9600
+#define MS_4            19200
+#define MS_5            38400
+#define MS_6            57600
+#define MS_7            115200
+#define MODEM_SPEED     MS_5				/* 38400 Baud set by default */
+#define MIN_BAUDRATE    150					/* min and max custom */
+#define MAX_BAUDRATE    1024000				/* Baud rates */
+#define MODEM_INIT	    "AT"				/* default init string */
+#define MODEM_RESET	    "ATZ"				/* default reset string */
+#define MODEM_CONNECT   "CONNECT"			/* default connect string */
+#define BSDCOMP         "bsdcomp"			/* pppd BSD compression option */
+#define DIAL_EXTRA      ""					/* default extra dialing chars */
+#define COMP_LEVEL      12					/* default S/W compression level */
+#define PPPD_SASYNCMAP  "20A0000"			/* pppd asyncmap for software */
+#define PPPD_HASYNCMAP  "00"				/* and hardware flow contol */
+#define PPPD_ESCAPE     "FF"				/* default pppd escape list */
+#define DEFLCOMP        "deflate"			/* pppd deflate compression opt */
 
 /* Manual terminal size defines */
 
@@ -154,6 +172,8 @@
 #define CB_NT_RAS		0x00020000
 #define DEFL_COMPRESS	0x00040000
 #define MODEM_ISDNDIAL	0x00080000
+#define AUTO_DNS		0x00100000
+#define DEFAULT_DOMAIN	0x00200000
 
 /* Default operating mode */
 
@@ -170,7 +190,8 @@ typedef struct {
 	char rname[MAXLEN_UNR+1];					/* and remotename pppd arg */
 	unsigned char maxAttempts;					/* maximum dialing attempts */
 	unsigned char sleepDelay;					/* delay between dials */
-	unsigned char connectWait;					/* maximum wait for connect */
+	unsigned char connectWait;					/* modem connection timeout */
+	unsigned char LCPWait;						/* net connection timeout */
 	unsigned char numPhones;					/* # of phone-number entries */
 	char phone[MAXNUM_TELS][MAXLEN_PHONE+1];	/* phone number table */
 	unsigned char numSlines;					/* # of script lines */
@@ -184,7 +205,7 @@ typedef struct {
 	unsigned char CBtermH;						/* for call-back connections */
 	char CBphone[MAXLEN_PHONE+1];				/* NT-RAS call-back phone # */
 	char modemDevice[MAXLEN_DEVICE+1];			/* modem device path */
-	unsigned char modemSpeed;					/* DTE<->DCE speed */
+	unsigned int modemSpeed;					/* DTE<->DCE Baud rate */
 	char modemReset[MAXLEN_MDMCMD+1];			/* modem reset command */
 	char modemInit[MAXLEN_MDMCMD+1];			/* modem init command */
 	char modemConnect[MAXLEN_MDMCMD+1];			/* modem connect string */
@@ -200,14 +221,18 @@ typedef struct {
 	unsigned char dns2[4];						/* secondary DNS entry */
 	unsigned int mtu;							/* the MTU */
 	unsigned int mru;							/* and the MRU */
+	char domainname[MAXLEN_DNNAME+1];			/* default domain name */
+	unsigned char ispPTT;						/* PTT selected for ISP */
+	unsigned char ispZone;						/* and its default zone */
 } xisprc_t;
 
 #define XISPRC_DEFAULT {{0},{0},{0},{0},{0},MAXNUM_RETRY,MAXSEC_DELAY,\
-						MAXSEC_CNWAIT,0,{{0}},0,{{0}},MAXSEC_CBDELAY,0,{{0}},\
-						TERMW,TERMH,MODEM_DEVICE,MODEM_SPEED,MODEM_RESET,\
-						MODEM_INIT,OPER_OPTS,DIAL_EXTRA,COMP_LEVEL,\
-						PPPD_HASYNCMAP,PPPD_ESCAPE,LOCAL_IP,REMOTE_IP,\
-						NETMASK,DNS,DNS,MTU,MRU}
+						MAXSEC_CNWAIT,MAXSEC_LCPWAIT,0,{{0}},0,{{0}},\
+						MAXSEC_CBDELAY,TERMW,TERMH,MODEM_DEVICE,\
+						MODEM_SPEED,MODEM_RESET,MODEM_INIT,OPER_OPTS,\
+						DIAL_EXTRA,COMP_LEVEL,PPPD_HASYNCMAP,PPPD_ESCAPE,\
+						LOCAL_IP,REMOTE_IP,NETMASK,DNS,DNS,MTU,MRU,{0},\
+						PTT_GR_EPAK,0}
 
 /* Logging mode options flags */
 
@@ -216,6 +241,13 @@ typedef struct {
 #define LOG_MONTHLY		0x0004
 #define LOG_BIMONTHLY	0x0008
 #define COST_READOUT	0x0010
+#define LOG_HINTS		0x1000
+
+/* Default logging mode */
+
+#define LOG_OPTS (LOG_NONE|LOG_HINTS)
+
+/* Associated strings */
 
 #define RTIME_STR		" Time\n On-Line:"
 #define RCOST_STR		" Cost\n Charged:"
@@ -228,18 +260,18 @@ typedef struct {
 	unsigned char numISPs;			/* number of ISPs in .xisprc */
 	unsigned char dfltISP;			/* default ISP to select after startup */
 	unsigned char numPTTs;			/* total number of PTTs in database */
-	unsigned char dfltPTT;			/* PTT selected for our QTH */
-	unsigned char dfltZone;			/* default Zone for selected PTT */
+	unsigned char costPTT;			/* total cost currency def'ed in this PTT */
 	unsigned short logOpts;			/* logging options flags */
-	char pppdPath[MAXLEN_PATH+1];	/* path to pppd daemon */
-	char runPath[MAXLEN_PATH+1];	/* path to pppd process ID files */
+	char pppdPath[MAXLEN_PATH+1];	/* path to the pppd daemon */
+	char runPath[MAXLEN_PATH+1];	/* path to the pppd process ID files */
+	char lockPath[MAXLEN_PATH+1];	/* path to the modem device lock file */
 	char chatPath[MAXLEN_PATH+1];	/* path to the chat utility */
 	char utilsPath[MAXLEN_PATH+1];	/* path to xispdial and xispterm */
 	char pipePath[MAXLEN_PATH+1];	/* path to the xisp named-pipe node */
 } glob_t;
 
-#define GLOB_DEFAULT {0, 0, MAXNUM_PTT, PTT_GR, 0, LOG_NONE, PPPD_PATH, \
-					  RUN_PATH, CHAT_PATH, UTILS_PATH, PIPE_PATH};
+#define GLOB_DEFAULT {0, 0, MAXNUM_PTT, PTT_GR_EPAK, LOG_OPTS, PPPD_PATH, \
+					  RUN_PATH, LOCK_PATH, CHAT_PATH, UTILS_PATH, PIPE_PATH};
 
 /* Connection mode basic types (ORed for combinations) */
 
