@@ -1,5 +1,5 @@
 /*
-  Copyright (C) 1997,1998,1999  Dimitrios P. Bouras
+  Copyright (C) 1997-2002  Dimitrios P. Bouras
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -20,7 +20,7 @@
 
 /* Constants relevant to the PTT data structures */
 
-#define MAXNUM_PTT      33	/* known PPT's for call cost calculation */
+#define MAXNUM_PTT      34	/* known PPT's for call cost calculation */
 #define MAXLEN_PTTNAME  32	/* characters reserved for PTT name */
 #define MAXLEN_CURRENCY 16	/* max length of national currency name */
 #define MAXNUM_ZONES    16	/* maximum number of price zones for a PTT */
@@ -28,10 +28,7 @@
 #define MAXNUM_CATEGORY 64	/* maximum number of categories within each zone */
 
 #define MAXVAL_MINUNITS		32		/* max value of minimum units */
-#define MAXVAL_MINCOST		100000	/* max value of minimum cost */
-#define MAXVAL_COSTQUANTUM	10000	/* max value of cost quantum */
 #define MAXNUM_DECIMALS		3		/* max number of decimals for printouts */
-#define MAXVAL_TARIFF		100000	/* maximum tariff value */
 #define MAXSEC_MINCOSTLEN	3600	/* max seconds of minimum cost length */
 #define MINVAL_RELDATE		-300	/* relative date minimun value */
 #define MAXVAL_RELDATE		+300	/* relative date maximun value */
@@ -71,6 +68,7 @@
 #define PTT_INETSERV	30	/* Inetservice, Germany */
 #define PTT_KOMTEL		31	/* Komtel, Germany */
 #define PTT_MOBILKOM	32	/* Mobilkom, Germany */
+#define PTT_IPY			32	/* IPY, Finland */
 
 /* PTT attribute masks */
 
@@ -133,11 +131,11 @@ typedef struct ptt_struct {
 	unsigned char min_units;					  /* minimum units charged */
 	float min_cost;								  /* minimum cost charged */
 	unsigned char charge_period;				  /* time charging period */
-	float cost_quantum;							  /* sec/unit or cost/minute */
+	float cost_quantum;							  /* cost/unit or cost/min */
 	char currency[MAXLEN_CURRENCY+1];			  /* local currency string */
 	unsigned char decimals;						  /* decimals for printouts */
 	float dflt_tariff[MAXNUM_ZONES];			  /* zone default tariffs */
-	unsigned int min_cost_len[MAXNUM_ZONES];	  /* min-cost length in sec */
+	float min_cost_len[MAXNUM_ZONES];			  /* min-cost length in sec */
 	char zone_name[MAXNUM_ZONES][MAXLEN_ZNAME+1]; /* zone names */
 	discount_t discount[MAXNUM_ZONES];			  /* special zone discount */
 	rule_t rule[MAXNUM_ZONES][MAXNUM_CATEGORY];   /* zone tariff rules */
@@ -147,10 +145,6 @@ typedef struct ptt_struct {
 					 {0}, 0, {0.0}, {0}, {{0}}, {{0.0,{0,0,0},{24,0,0}}}, \
 					 {{{RULE_WEEKALL,{0,1},{11,31},0,{0,0,0},{24,0,0},0.0}}}}
 
-typedef struct xispPTTs_struct {
-	
-} xispPTTs_t;
-
 /* Module interface routines */
 
 int initLogFnames(glob_t *global);
@@ -159,14 +153,14 @@ void logs_init(glob_t *global, char *logdirname,
 void costCalcReset(ptt_t *ptt, time_t upTime);
 float callCost(ptt_t* ptt, unsigned long onLineSecs,
 			   unsigned long *onLineUnits);
-void readXispCost(unsigned long *totalTime, unsigned long *totalUnits);
+void readXispCost(unsigned long *totalTime, float *totalCost);
 #define restartXispCost(a,b) readXispCost(a,b)
-void writeXispCost(unsigned long totalTime, unsigned long totalUnits);
+void writeXispCost(unsigned long totalTime, float totalCost);
 void writeXispLog(int up, ptt_t *ptt, char *ts, char *ISPn, char *IPs,
 				  char *speed, unsigned long onLineSecs, float onLineCost,
 				  unsigned long pTXed, unsigned long pRXed);
 int buildStats(glob_t *global, unsigned int *current,
-			   unsigned long *tblTime, unsigned long *tblUnits);
+			   unsigned long *tblTime, float *tblCosts);
 void initXispPTT(ptt_t *);
 void readXispPTTs(ptt_t **pttdataptr, glob_t *global);
 void writeXispPTTs(ptt_t *pttdata, glob_t *global);
