@@ -1,5 +1,5 @@
 /*
-  Copyright (C) 1997,1998,1999  Dimitrios P. Bouras
+  Copyright (C) 1997-2005  Dimitrios P. Bouras
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -23,13 +23,17 @@
 #include <stdio.h>
 #include <stdlib.h>
 #include <errno.h>
-#include <varargs.h>
+#include <stdarg.h>
 #include <string.h>
 #include <math.h>
 #include <sys/time.h>
+#include <ctype.h>
+#include <unistd.h>
+#include <sys/wait.h>
 #include "common.h"
 #include "logs.h"
 #include "version.h"
+#include "pcode.h"
 
 #ifdef SUNOS41x
 #include <memory.h>
@@ -37,7 +41,11 @@ extern int sys_nerr;
 extern char *sys_errlist[];
 extern int errno;
 extern int sscanf(), fputs(), vfprintf(), fscanf(), fprintf(), fclose();
+extern int rewind(), toupper(), system();
 extern long strtol();
+#else
+/* exit() replacement for emulating on_exit() functionality */
+#define exit(x) {extern int exitStatus; exitStatus = x; exit(x);}
 #endif
 
 /* Local storage */
@@ -53,17 +61,17 @@ static glob_t reset_global = GLOB_DEFAULT;	/* for resetting global struct */
 
 /* Convert binary password to ASCII printable characters */
 
-static char *passwd2ascii(unsigned char *pwd)
+static unsigned char *passwd2ascii(unsigned char *pwd)
 {
 	static unsigned char apwd[2*MAXLEN_PASSWD+1];
-	unsigned char msh, lsh, *p;
+	unsigned char up, lp, *p;
 	int i;
 
 	for (i=0, p=apwd; i<MAXLEN_PASSWD; i++, pwd++) {
-		msh = (*pwd & 0xF0) >> 4;
-		lsh = *pwd & 0x0F;
-		*p++ = lsh + 'A';
-		*p++ = msh + 'A';
+		lp = *pwd & 0x03F;
+		*p++ = lp + 33;
+		up = (*pwd & 0x0FC) >> 2;
+		*p++ = up + 161;
 	}
 	*p = 0;
 	return apwd;
@@ -80,55 +88,25 @@ static char *IP2str(unsigned char *ip)
 	return IPstr;
 }
 
-/* Convert speed bit-mask to ASCII string */
-
-static char *speed2str(unsigned char speed)
-{
-	static char sstr[8][8] = {"1200","2400","4800","9600",
-							  "19200","38400","57600","115200"};
-	static int snum[8] = {0x01,0x02,0x04,0x08,0x10,0x20,0x40,0x80};
-	int i;
-
-	for (i=0; i<8; i++)
-		if (speed & snum[i])
-			return sstr[i];
-	return NULL;
-}
-
 /* Convert ASCII password to binary */
 
-static char *ascii2passwd(char *buf)
+static unsigned char *ascii2passwd(unsigned char *buf)
 {
 	static unsigned char pwd[MAXLEN_PASSWD+1];
-	unsigned char msh, lsh, *p;
+	unsigned char up, lp;
 	int i = 0;
 
 	if (*buf) {
-		for (p=pwd; i<MAXLEN_PASSWD; i++) {
-			lsh = (*buf++ - 'A') & 0x0F;
-			msh = (*buf++ - 'A') & 0x0F;
-			pwd[i] = (msh << 4) | lsh;
+		for (; i<MAXLEN_PASSWD; i++) {
+			lp = (*buf++ - 33) & 0x00F;
+			up = (*buf++ - 161) & 0x03C;
+			pwd[i] = (up << 2) | lp;
 		}
 	}
 	pwd[i] = 0;
 	return pwd;
 }
 
-/* Convert ASCII speed string to bit-mask */
-
-static unsigned int str2speed(char *buf)
-{
-	static char sstr[8][8] = {"1200","2400","4800","9600",
-							  "19200","38400","57600","115200"};
-	static int snum[8] = {0x01,0x02,0x04,0x08,0x10,0x20,0x40,0x80};
-	int i;
-
-	for (i=0; i<8; i++)
-		if (strcmp(buf, sstr[i]) == 0)
-			return snum[i];
-	return 0;
-}
-
 /* Convert ASCII string to IP address */
 
 static unsigned char *str2IP(char *buf)
@@ -169,15 +147,13 @@ static void doErr(char *msg)
 
 /* Function for writing strings in the .xisprc resource control file */
 
-static int rcPrint(va_alist) va_dcl
+static int rcPrint(FILE *fp, ...)
 {
 	int bw;
 	va_list ap;
 	char *fmt;
-	FILE *fp;
 
-	va_start(ap);
-	fp = va_arg(ap, FILE*);
+	va_start(ap, fp);
 	fmt = va_arg(ap, char*);
 	bw = vfprintf(fp, fmt, ap);
 	va_end(ap);
@@ -191,68 +167,82 @@ static int rcPrint(va_alist) va_dcl
 static void outputAllXisprc(xisprc_t *p, FILE *rcfp, glob_t *global)
 {
 	int i, n;
+	char tmp[2*MAXLEN_PASSWD+1];
 
 	rcPrint(rcfp, "xISP-%s\n", Version);
-	rcPrint(rcfp, "NUMISPS: %d\n", global->numISPs);
-	rcPrint(rcfp, "DEFAULTISP: %d\n", global->dfltISP);
-	rcPrint(rcfp, "NUMPTTS: %d\n", global->numPTTs);
-	rcPrint(rcfp, "QTHPTT: %d\n", global->dfltPTT);
-	rcPrint(rcfp, "QTHZONE: %d\n", global->dfltZone);
-	rcPrint(rcfp, "LOGOPTS: %hX\n", global->logOpts);
-	rcPrint(rcfp, "PPPDPATH: %s\n", global->pppdPath);
-	rcPrint(rcfp, "RUNPATH: %s\n", global->runPath);
-	rcPrint(rcfp, "CHATPATH: %s\n", global->chatPath);
-	rcPrint(rcfp, "UTILSPATH: %s\n", global->utilsPath);
-	rcPrint(rcfp, "PIPEPATH: %s\n", global->pipePath);
+	rcPrint(rcfp, "NumISPs: %d\n", global->numISPs);
+	rcPrint(rcfp, "DefaultISP: %d\n", global->dfltISP);
+	rcPrint(rcfp, "NumPTTs: %d\n", global->numPTTs);
+	rcPrint(rcfp, "CostPTT: %d\n", global->costPTT);
+	rcPrint(rcfp, "LogOpts: %hX\n", global->logOpts);
+	rcPrint(rcfp, "PppdPath: %s\n", global->pppdPath);
+	rcPrint(rcfp, "RunPath: %s\n", global->runPath);
+	rcPrint(rcfp, "LockPath: %s\n", global->lockPath);
+	rcPrint(rcfp, "ChatPath: %s\n", global->chatPath);
+	rcPrint(rcfp, "UtilsPath: %s\n", global->utilsPath);
+	rcPrint(rcfp, "PipePath: %s\n", global->pipePath);
 	for (i=0; p!=NULL && i<global->numISPs; i++, p++) {
-		rcPrint(rcfp, "-%d-\n", i);
-		rcPrint(rcfp, "DESCR: %s\n", p->descr);
-		rcPrint(rcfp, "ACCOUNT: %s\n", p->account);
-		if (*(p->account))
-			rcPrint(rcfp, "PASSWD: %s\n", passwd2ascii(p->passwd));
+		rcPrint(rcfp, "[%d]\n", i);
+		rcPrint(rcfp, "Description: %s\n", p->descr);
+		rcPrint(rcfp, "Account: %s\n", p->account);
+		if (*(p->account)) {
+			memset(tmp, 0, 2*MAXLEN_PASSWD+1);
+			pdecode(tmp, p->passwd);
+			n = strlen(tmp);
+			if (n % 8)
+				n = 8*((n/8) + 1);
+			memset(tmp, 0, 2*MAXLEN_PASSWD+1);
+			memcpy(tmp, passwd2ascii(p->passwd), 2*n);
+			tmp[2*n] = 0;
+			rcPrint(rcfp, "Password: %s\n", tmp);
+		}
 		else
-			rcPrint(rcfp, "PASSWD: \n");
-		rcPrint(rcfp, "NAME: %s\n", p->name);
-		rcPrint(rcfp, "REMOTENAME: %s\n", p->rname);
-		rcPrint(rcfp, "MAXATTEMPTS: %d\n", p->maxAttempts);
-		rcPrint(rcfp, "SLEEPDELAY: %d\n", p->sleepDelay);
-		rcPrint(rcfp, "CONNECTWAIT: %d\n", p->connectWait);
-		rcPrint(rcfp, "NUMPHONES: %d\n", p->numPhones);
+			rcPrint(rcfp, "Password: \n");
+		rcPrint(rcfp, "Name: %s\n", p->name);
+		rcPrint(rcfp, "RemoteName: %s\n", p->rname);
+		rcPrint(rcfp, "MaxAttempts: %d\n", p->maxAttempts);
+		rcPrint(rcfp, "SleepDelay: %d\n", p->sleepDelay);
+		rcPrint(rcfp, "ConnectWait: %d\n", p->connectWait);
+		rcPrint(rcfp, "LCPWait: %d\n", p->LCPWait);
+		rcPrint(rcfp, "NumPhones: %d\n", p->numPhones);
 		for (n=0; n<p->numPhones; n++)
-			rcPrint(rcfp, " PHONE%d: %s\n", n, p->phone[n]);
-		rcPrint(rcfp, "NUMSLINES: %d\n", p->numSlines);
+			rcPrint(rcfp, " Phone[%d]: %s\n", n, p->phone[n]);
+		rcPrint(rcfp, "NumSLines: %d\n", p->numSlines);
 		for (n=0; n<p->numSlines; n++) {
-			rcPrint(rcfp, " SLINE%dE: %s\n", n, p->sline[n]);
-			rcPrint(rcfp, " SLINE%dS: %s\n", n, p->sline[MAXNUM_SLINES+n]);
+			rcPrint(rcfp, " SLine[%d]E: %s\n", n, p->sline[n]);
+			rcPrint(rcfp, " SLine[%d]S: %s\n", n, p->sline[MAXNUM_SLINES+n]);
 		}
-		rcPrint(rcfp, "CBDELAY: %d\n", p->CBDelay);
-		rcPrint(rcfp, "NUMCBSLINES: %d\n", p->numCBSlns);
+		rcPrint(rcfp, "CBDelay: %d\n", p->CBDelay);
+		rcPrint(rcfp, "NumCBSLines: %d\n", p->numCBSlns);
 		for (n=0; n<p->numCBSlns; n++) {
-			rcPrint(rcfp, " CBSLINE%dE: %s\n", n, p->CBsln[n]);
-			rcPrint(rcfp, " CBSLINE%dS: %s\n", n, p->CBsln[MAXNUM_SLINES+n]);
+			rcPrint(rcfp, " CBSLine[%d]E: %s\n", n, p->CBsln[n]);
+			rcPrint(rcfp, " CBSLine[%d]S: %s\n", n, p->CBsln[MAXNUM_SLINES+n]);
 		}
-		rcPrint(rcfp, "TERMW: %d\n", p->termW);
-		rcPrint(rcfp, "TERMH: %d\n", p->termH);
-		rcPrint(rcfp, "CBTERMW: %d\n", p->CBtermW);
-		rcPrint(rcfp, "CBTERMH: %d\n", p->CBtermH);
-		rcPrint(rcfp, "CBPHONE: %s\n", p->CBphone);
-		rcPrint(rcfp, "MODEMDEVICE: %s\n", p->modemDevice);
-		rcPrint(rcfp, "MODEMSPEED: %s\n", speed2str(p->modemSpeed));
-		rcPrint(rcfp, "MODEMRESET: %s\n", p->modemReset);
-		rcPrint(rcfp, "MODEMINIT: %s\n", p->modemInit);
-		rcPrint(rcfp, "MODEMCONNECT: %s\n", p->modemConnect);
-		rcPrint(rcfp, "OPEROPTS: %lX\n", p->operOpts);
-		rcPrint(rcfp, "DIALEXTRA: %s\n", p->dialExtra);
-		rcPrint(rcfp, "COMPLEVEL: %d\n", p->compLevel);
-		rcPrint(rcfp, "ASYNCMAP: %s\n", p->asyncmap);
-		rcPrint(rcfp, "ESCAPE: %s\n", p->escape);
-		rcPrint(rcfp, "LOCALIP: %s\n", IP2str(p->localIP));
-		rcPrint(rcfp, "REMOTEIP: %s\n", IP2str(p->remoteIP));
-		rcPrint(rcfp, "NETMASK: %s\n", IP2str(p->netmask));
+		rcPrint(rcfp, "TermW: %d\n", p->termW);
+		rcPrint(rcfp, "TermH: %d\n", p->termH);
+		rcPrint(rcfp, "CBTermW: %d\n", p->CBtermW);
+		rcPrint(rcfp, "CBTermH: %d\n", p->CBtermH);
+		rcPrint(rcfp, "CBPhone: %s\n", p->CBphone);
+		rcPrint(rcfp, "ModemDevice: %s\n", p->modemDevice);
+		rcPrint(rcfp, "ModemSpeed: %u\n", p->modemSpeed);
+		rcPrint(rcfp, "ModemReset: %s\n", p->modemReset);
+		rcPrint(rcfp, "ModemInit: %s\n", p->modemInit);
+		rcPrint(rcfp, "ModemConnect: %s\n", p->modemConnect);
+		rcPrint(rcfp, "OperOpts: %lX\n", p->operOpts);
+		rcPrint(rcfp, "DialExtra: %s\n", p->dialExtra);
+		rcPrint(rcfp, "CompLevel: %d\n", p->compLevel);
+		rcPrint(rcfp, "AsyncMap: %s\n", p->asyncmap);
+		rcPrint(rcfp, "Escape: %s\n", p->escape);
+		rcPrint(rcfp, "LocalIP: %s\n", IP2str(p->localIP));
+		rcPrint(rcfp, "RemoteIP: %s\n", IP2str(p->remoteIP));
+		rcPrint(rcfp, "NetMask: %s\n", IP2str(p->netmask));
 		rcPrint(rcfp, "DNS1: %s\n", IP2str(p->dns1));
 		rcPrint(rcfp, "DNS2: %s\n", IP2str(p->dns2));
 		rcPrint(rcfp, "MTU: %u\n", p->mtu);
 		rcPrint(rcfp, "MRU: %u\n", p->mru);
+		rcPrint(rcfp, "DomainName: %s\n", p->domainname);
+		rcPrint(rcfp, "ISPPTT: %d\n", p->ispPTT);
+		rcPrint(rcfp, "ISPZone: %d\n", p->ispZone);
 	}
 }
 
@@ -263,6 +253,7 @@ static char *rcVersion(FILE *fp)
 {
 	static char version[8] = {0};
 
+	rewind(fp);
 	if (fscanf(fp, "xISP-%7[.0-9] ", version) < 1) {
 		fputs("xISP: rcVersion: error reading file version!\n", stderr);
 		exit(1);
@@ -277,7 +268,7 @@ static void rcISP(FILE *fp, int nISP)
 {
 	int n;
 
-	if (fscanf(fp, "-%3d- ", &n) < 1) {
+	if (fscanf(fp, "[%3d] ", &n) < 1) {
 		fprintf(stderr, "xISP: rcISP: error reading ISP number in %s\n",
 				rcfn);
 		exit(1);
@@ -309,8 +300,8 @@ static void rcLineError(char *function, char *line)
 static char *readParam(FILE *fp, char *name, char type,
 					   int llimit, int ulimit, void *data)
 {
-	static char line[MAXLEN_LINE];
-	char sparam[MAXLEN_PARAM] = {0}, *p, *endp;
+	static char line[MAXLEN_LINE], NAME[MAXLEN_PARAM];
+	char sparam[MAXLEN_PARAM] = {0}, *p, *P, *endp;
 	long iparam;
 	int len, hex = 0;
 
@@ -319,9 +310,14 @@ static char *readParam(FILE *fp, char *name, char type,
 		exit(1);
 	}
 	line[strlen(line)-1] = 0;
+	for (p=name, P=NAME; *p; p++, P++)
+		*P = toupper(*p);
+	*P = 0;
 	if ((p=strchr(line,':')) == NULL)
 		rcLineError("readParam", line);
-	if (strncmp(name, line, (int)(p-line))) {
+	if (strncmp(name, line, (int)(p-line)) &&	/* accept all capitals */
+		strncmp(NAME, line, (int)(p-line)))
+	{
 		fprintf(stderr, "xISP: readParam: expected [%s], got [%s]\n",
 				name, line);
 		exit(1);
@@ -333,7 +329,8 @@ static char *readParam(FILE *fp, char *name, char type,
 			len = strlen(sparam);
 			if (ulimit<len || len<llimit)
 				rcLineError("readParam", line);
-			strcpy((char *)data, sparam);
+			strncpy((char *)data, sparam, len);
+			((char *)data)[len] = 0;
 			break;
 
 		case 'X':
@@ -343,14 +340,18 @@ static char *readParam(FILE *fp, char *name, char type,
 		case 'I':
 			iparam = strtol(sparam, &endp, (hex)? 16:10);
 			if (hex) {
-				if (type == 'X') *((unsigned short *)data) = iparam;
-				else *((unsigned long *)data) = iparam;
+				if (type == 'X')
+					*((unsigned short *)data) = iparam;
+				else
+					*((unsigned long *)data) = iparam;
 			}
 			else {
 				if (ulimit<iparam || iparam<llimit || endp == sparam || *endp)
 					rcLineError("readParam", line);
-				if (type == 'B') *((unsigned char *)data) = iparam;
-				else *((unsigned int *)data) = iparam;
+				if (type == 'B')
+					*((unsigned char *)data) = iparam;
+				else
+					*((unsigned int *)data) = iparam;
 			}
 			break;
 
@@ -363,17 +364,17 @@ static char *readParam(FILE *fp, char *name, char type,
 
 static void inputGlobals(FILE *rcfp, glob_t *global)
 {
-	readParam(rcfp, "NUMISPS", 'B', 0, 255, &(global->numISPs));
-	readParam(rcfp, "DEFAULTISP", 'B', 0, 255, &(global->dfltISP));
-	readParam(rcfp, "NUMPTTS", 'B', 1, 255, &(global->numPTTs));
-	readParam(rcfp, "QTHPTT", 'B', 0, (global->numPTTs)-1, &(global->dfltPTT));
-	readParam(rcfp, "QTHZONE", 'B', 0, MAXNUM_ZONES-1, &(global->dfltZone));
-	readParam(rcfp, "LOGOPTS", 'X', 0, 0, &(global->logOpts));
-	readParam(rcfp, "PPPDPATH", 'S', 0, MAXLEN_PATH, global->pppdPath);
-	readParam(rcfp, "RUNPATH", 'S', 0, MAXLEN_PATH, global->runPath);
-	readParam(rcfp, "CHATPATH", 'S', 0, MAXLEN_PATH, global->chatPath);
-	readParam(rcfp, "UTILSPATH", 'S', 0, MAXLEN_PATH, global->utilsPath);
-	readParam(rcfp, "PIPEPATH", 'S', 0, MAXLEN_PATH, global->pipePath);
+	readParam(rcfp, "NumISPs", 'B', 0, 255, &(global->numISPs));
+	readParam(rcfp, "DefaultISP", 'B', 0, 255, &(global->dfltISP));
+	readParam(rcfp, "NumPTTs", 'B', 1, 255, &(global->numPTTs));
+	readParam(rcfp, "CostPTT", 'B', 0, global->numPTTs-1, &(global->costPTT));
+	readParam(rcfp, "LogOpts", 'X', 0, 0, &(global->logOpts));
+	readParam(rcfp, "PppdPath", 'S', 0, MAXLEN_PATH, global->pppdPath);
+	readParam(rcfp, "RunPath", 'S', 0, MAXLEN_PATH, global->runPath);
+	readParam(rcfp, "LockPath", 'S', 0, MAXLEN_PATH, global->lockPath);
+	readParam(rcfp, "ChatPath", 'S', 0, MAXLEN_PATH, global->chatPath);
+	readParam(rcfp, "UtilsPath", 'S', 0, MAXLEN_PATH, global->utilsPath);
+	readParam(rcfp, "PipePath", 'S', 0, MAXLEN_PATH, global->pipePath);
 }
 
 /* Function for reading in the ISP sections from the .xisprc file */
@@ -382,73 +383,82 @@ static void inputAllXisprc(xisprc_t *p, FILE *rcfp, glob_t *global)
 {
 	int i, j, n;
 	char buf[2*MAXLEN_PASSWD+1], pname[32] = {0}, *line;
-	unsigned char *ipp;
+	unsigned char *ipp, tmp[2*MAXLEN_PASSWD+1], epasswd[MAXLEN_PASSWD+1];
 
 	for (i=0; i<global->numISPs; i++, p++) {
 		memset(p, 0, sizeof(xisprc_t));
 		rcISP(rcfp, i);
-		readParam(rcfp, "DESCR", 'S', 0, MAXLEN_DESCR, p->descr);
-		readParam(rcfp, "ACCOUNT", 'S', 0, MAXLEN_ACCOUNT, p->account);
+		readParam(rcfp, "Description", 'S', 0, MAXLEN_DESCR, p->descr);
+		readParam(rcfp, "Account", 'S', 0, MAXLEN_ACCOUNT, p->account);
 		if (! *(p->account))
-			n = 0;
-		else
-			n = 2*MAXLEN_PASSWD;
-		readParam(rcfp, "PASSWD", 'S', n, n, buf);
-		memcpy(p->passwd, ascii2passwd(buf), MAXLEN_PASSWD);
-		readParam(rcfp, "NAME", 'S', 0, MAXLEN_UNR, p->name);
-		readParam(rcfp, "REMOTENAME", 'S', 0, MAXLEN_UNR, p->rname);
-		readParam(rcfp, "MAXATTEMPTS", 'B', 0, 255, &(p->maxAttempts));
-		readParam(rcfp, "SLEEPDELAY", 'B', 0, 255, &(p->sleepDelay));
-		readParam(rcfp, "CONNECTWAIT", 'B', 0, 255, &(p->connectWait));
-		readParam(rcfp, "NUMPHONES", 'B', 0, MAXNUM_TELS, &(p->numPhones));
+			readParam(rcfp, "Password", 'S', 0, 0, buf);
+		else {
+			readParam(rcfp, "Password", 'S', 0, 2*MAXLEN_PASSWD, buf);
+			n = strlen(buf);
+			if (n < 2*MAXLEN_PASSWD) {
+				memset(tmp, 0, 2*MAXLEN_PASSWD+1);
+				pencode(epasswd, tmp);
+				memcpy(tmp, passwd2ascii(epasswd), 2*MAXLEN_PASSWD);
+				memcpy(buf+n, tmp+n, 2*MAXLEN_PASSWD-n);
+				buf[2*MAXLEN_PASSWD] = 0;
+			}
+		}
+		memcpy(p->passwd, ascii2passwd((unsigned char *)buf), MAXLEN_PASSWD);
+		readParam(rcfp, "Name", 'S', 0, MAXLEN_UNR, p->name);
+		readParam(rcfp, "RemoteName", 'S', 0, MAXLEN_UNR, p->rname);
+		readParam(rcfp, "MaxAttempts", 'B', 0, 255, &(p->maxAttempts));
+		readParam(rcfp, "SleepDelay", 'B', 0, 255, &(p->sleepDelay));
+		readParam(rcfp, "ConnectWait", 'B',MINSEC_MODEM_TO,255,
+				  &(p->connectWait));
+		readParam(rcfp, "LCPWait", 'B', MINSEC_LINK_TO, 255, &(p->LCPWait));
+		readParam(rcfp, "NumPhones", 'B', 0, MAXNUM_TELS, &(p->numPhones));
 		for (j=0; j<p->numPhones; j++) {
-			sprintf(pname, " PHONE%d", j);
+			sprintf(pname, " Phone[%d]", j);
 			readParam(rcfp, pname, 'S', 0, MAXLEN_PHONE, p->phone[j]);
 		}
-		readParam(rcfp, "NUMSLINES", 'B', 0, MAXNUM_SLINES, &(p->numSlines));
+		readParam(rcfp, "NumSLines", 'B', 0, MAXNUM_SLINES, &(p->numSlines));
 		for (j=0; j<p->numSlines; j++) {
-			sprintf(pname, " SLINE%dE", j);
+			sprintf(pname, " SLine[%d]E", j);
 			readParam(rcfp, pname, 'S', 0, MAXLEN_SLINE, p->sline[j]);
-			sprintf(pname, " SLINE%dS", j);
+			sprintf(pname, " SLine[%d]S", j);
 			readParam(rcfp, pname, 'S', 0, MAXLEN_SLINE,
 					  p->sline[MAXNUM_SLINES+j]);
 		}
-		readParam(rcfp, "CBDELAY", 'B', 0, 255, &(p->CBDelay));
-		readParam(rcfp, "NUMCBSLINES", 'B', 0, MAXNUM_SLINES, &(p->numCBSlns));
+		readParam(rcfp, "CBDelay", 'B', 0, 255, &(p->CBDelay));
+		readParam(rcfp, "NumCBSLines", 'B', 0, MAXNUM_SLINES, &(p->numCBSlns));
 		for (j=0; j<p->numCBSlns; j++) {
-			sprintf(pname, " CBSLINE%dE", j);
+			sprintf(pname, " CBSLine[%d]E", j);
 			readParam(rcfp, pname, 'S', 0, MAXLEN_SLINE, p->CBsln[j]);
-			sprintf(pname, " CBSLINE%dS", j);
+			sprintf(pname, " CBSLine[%d]S", j);
 			readParam(rcfp, pname, 'S', 0, MAXLEN_SLINE,
 					  p->CBsln[MAXNUM_SLINES+j]);
 		}
-		readParam(rcfp, "TERMW", 'B', MINCHAR_TERMW, 255, &(p->termW));
-		readParam(rcfp, "TERMH", 'B', MINCHAR_TERMH, 255, &(p->termH));
-		readParam(rcfp, "CBTERMW", 'B', MINCHAR_TERMW, 255, &(p->CBtermW));
-		readParam(rcfp, "CBTERMH", 'B', MINCHAR_TERMH, 255, &(p->CBtermH));
-		readParam(rcfp, "CBPHONE", 'S', 0, MAXLEN_PHONE, p->CBphone);
-		readParam(rcfp, "MODEMDEVICE", 'S', 0, MAXLEN_DEVICE, p->modemDevice);
-		line = readParam(rcfp, "MODEMSPEED", 'S', 4, 6, buf);
-		if (! (n=str2speed(buf)))
-			rcLineError("inputAllXisprc", line);
-		p->modemSpeed = n;
-		readParam(rcfp, "MODEMRESET", 'S', 0, MAXLEN_MDMCMD, p->modemReset);
-		readParam(rcfp, "MODEMINIT", 'S', 0, MAXLEN_MDMCMD, p->modemInit);
-		readParam(rcfp, "MODEMCONNECT", 'S',0, MAXLEN_MDMSTR, p->modemConnect);
-		readParam(rcfp, "OPEROPTS", 'L', 0, 0, &(p->operOpts));
-		readParam(rcfp, "DIALEXTRA", 'S', 0, MAXLEN_DIALEXTRA, p->dialExtra);
-		readParam(rcfp, "COMPLEVEL", 'B', 9, 15, &(p->compLevel));
-		readParam(rcfp, "ASYNCMAP", 'S', 2, MAXDIG_ASYNCMAP, p->asyncmap);
-		readParam(rcfp, "ESCAPE", 'S', 0, MAXLEN_ESCAPE, p->escape);
-		line = readParam(rcfp, "LOCALIP", 'S', 7, MAXLEN_IP, buf);
+		readParam(rcfp, "TermW", 'B', MINCHAR_TERMW, 255, &(p->termW));
+		readParam(rcfp, "TermH", 'B', MINCHAR_TERMH, 255, &(p->termH));
+		readParam(rcfp, "CBTermW", 'B', MINCHAR_TERMW, 255, &(p->CBtermW));
+		readParam(rcfp, "CBTermH", 'B', MINCHAR_TERMH, 255, &(p->CBtermH));
+		readParam(rcfp, "CBPhone", 'S', 0, MAXLEN_PHONE, p->CBphone);
+		readParam(rcfp, "ModemDevice", 'S', 0, MAXLEN_DEVICE, p->modemDevice);
+		readParam(rcfp, "ModemSpeed", 'I', MIN_BAUDRATE, MAX_BAUDRATE,
+				  &(p->modemSpeed));
+		readParam(rcfp, "ModemReset", 'S', 0, MAXLEN_MDMCMD, p->modemReset);
+		readParam(rcfp, "ModemInit", 'S', 0, MAXLEN_MDMCMD, p->modemInit);
+		readParam(rcfp, "ModemConnect", 'S',0, MAXLEN_MDMSTR, p->modemConnect);
+		readParam(rcfp, "OperOpts", 'L', 0, 0, &(p->operOpts));
+		readParam(rcfp, "DialExtra", 'S', 0, MAXLEN_DIALEXTRA, p->dialExtra);
+		readParam(rcfp, "CompLevel", 'B', MIN_COMPLEVEL, MAX_COMPLEVEL,
+				  &(p->compLevel));
+		readParam(rcfp, "AsyncMap", 'S', 2, MAXDIG_ASYNCMAP, p->asyncmap);
+		readParam(rcfp, "Escape", 'S', 0, MAXLEN_ESCAPE, p->escape);
+		line = readParam(rcfp, "LocalIP", 'S', 7, MAXLEN_IP, buf);
 		if ((ipp=str2IP(buf)) == NULL)
 			rcLineError("inputAllXisprc", line);
 		memcpy(p->localIP, ipp, 4);
-		line = readParam(rcfp, "REMOTEIP", 'S', 7, MAXLEN_IP, buf);
+		line = readParam(rcfp, "RemoteIP", 'S', 7, MAXLEN_IP, buf);
 		if ((ipp=str2IP(buf)) == NULL)
 			rcLineError("inputAllXisprc", line);
 		memcpy(p->remoteIP, ipp, 4);
-		line = readParam(rcfp, "NETMASK", 'S', 7, MAXLEN_IP, buf);
+		line = readParam(rcfp, "NetMask", 'S', 7, MAXLEN_IP, buf);
 		if ((ipp=str2IP(buf)) == NULL)
 			rcLineError("inputAllXisprc", line);
 		memcpy(p->netmask, ipp, 4);
@@ -462,6 +472,9 @@ static void inputAllXisprc(xisprc_t *p, FILE *rcfp, glob_t *global)
 		memcpy(p->dns2, ipp, 4);
 		readParam(rcfp, "MTU", 'I', 128, 2048, &(p->mtu));
 		readParam(rcfp, "MRU", 'I', 128, 2048, &(p->mru));
+		readParam(rcfp, "DomainName", 'S', 0, MAXLEN_DNNAME, p->domainname);
+		readParam(rcfp, "ISPPTT", 'B', 0, (global->numPTTs)-1, &(p->ispPTT));
+		readParam(rcfp, "ISPZone", 'B', 0, MAXNUM_ZONES-1, &(p->ispZone));
 	}
 }
 
@@ -492,6 +505,7 @@ void initXisprc(xisprc_t *p)
 	p->maxAttempts = MAXNUM_RETRY;
 	p->sleepDelay = MAXSEC_DELAY;
 	p->connectWait = MAXSEC_CNWAIT;
+	p->LCPWait = MAXSEC_LCPWAIT;
 	p->numPhones = 0;
 	for (i=0; i<MAXNUM_TELS; i++)
 		p->phone[i][0] = 0;
@@ -521,6 +535,9 @@ void initXisprc(xisprc_t *p)
 	memcpy(p->dns2, DNS, 4);
 	p->mtu = MTU;
 	p->mru = MRU;
+	memset(p->domainname, 0, MAXLEN_DNNAME);
+	p->ispPTT = PTT_GR_EPAK;
+	p->ispZone = 0;
 }
 
 /* Read all xisprc data records from the user file, or create
@@ -529,15 +546,42 @@ void initXisprc(xisprc_t *p)
 void readXisprc(char *rcfname, xisprc_t **rcdataptr, glob_t *global)
 {
 	FILE *rcfp;
-	extern void outofMem();
+	extern void outofMem(), alertMessage();
+	extern int actionVerify();
+	char msg[512] = {0}, *p;
+	int status;
 
 	rcfn = rcfname;
 	rcfp = fopen(rcfn, "r");
 	if (rcfp != NULL) {
-		if (strcmp(rcVersion(rcfp), Version)) {
-			fprintf(stderr, "xISP: %s is not a v%s file!\n", rcfn, Version);
-			fprintf(stderr, "xISP: Please run xisprccv with no arguments.\n");
-			exit(1);
+		if (strcmp(p=rcVersion(rcfp), Version)) {
+			sprintf(msg, "%s claims to be version %s,\nwhile %s "
+					"is needed. Do you want to invoke xisprccv to\n"
+					"convert the file?", rcfn, p, Version);
+			if (! actionVerify(msg, 0)) {
+				alertMessage("Exiting:", 0, 1, "OK. Aborting...");
+				exit(0);
+			}
+			fclose(rcfp);
+			if ((status=system("xisprccv"), status=WEXITSTATUS(status)) ||
+				(rcfp = fopen(rcfn, "r")) == NULL ||
+				strcmp(rcVersion(rcfp), Version))
+			{
+				if (status == 2) {
+					alertMessage("Exiting:", 0, 0,
+								 "Conversion of your .xisprc succeeded, but "
+								 "your\nlog files seem to be out of order! "
+								 "  Please rename\n$HOME/.xisplogs to "
+								 "$HOME/.xisplogs.old so that\nnew logs can "
+								 "be created!");
+					exit(0);
+				}
+				else {
+					alertMessage("Exiting:", 0, 1,
+								 "Conversion failed! Aborting...");
+					exit(1);
+				}
+			}
 		}
 		*global = reset_global;
 		inputGlobals(rcfp, global);
@@ -545,6 +589,8 @@ void readXisprc(char *rcfname, xisprc_t **rcdataptr, glob_t *global)
 		if (*rcdataptr == NULL)
 			outofMem();
 		initXisprc(*rcdataptr);	/* initialize default record */
+		if ((*rcdataptr)[0].ispPTT >= global->numPTTs)
+			(*rcdataptr)[0].ispPTT = global->numPTTs-1;
 		inputAllXisprc(&((*rcdataptr)[1]), rcfp, global);
 		fclose(rcfp);
 	}
