@@ -1,5 +1,5 @@
 /*
-  Copyright (C) 1997,1998,1999  Dimitrios P. Bouras
+  Copyright (C) 1997-2002  Dimitrios P. Bouras
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -18,7 +18,7 @@
    For author contact information, look in the README file.
 */
 
-static char RCSid[] = "$Id: version.h,v 2.6.1.1 1999/08/26 09:27:37 dimitri Exp $"; 
-static char Version[] = "2.6";
-static char PatchLevel[] = "p1";
+static char RCSid[] = "$Id: version.h,v 2.7 2001/11/24 12:45:22 dimitri Exp dimitri $"; 
+static char Version[] = "2.7";
+static char PatchLevel[] = "";
 
