@@ -1,5 +1,5 @@
 /*
-  Copyright (C) 1997,1998,1999  Dimitrios P. Bouras
+  Copyright (C) 1997-2002  Dimitrios P. Bouras
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -26,7 +26,7 @@ void doConnect(FL_OBJECT *, long);
 void doInterrupt(FL_OBJECT *, long);
 void doDisconnect(FL_OBJECT *, long);
 void doQuit(FL_OBJECT *, long);
-void doOptions(FL_OBJECT *, long);
+void doFile(FL_OBJECT *, long);
 void doHelp(FL_OBJECT *, long);
 void doISPDropChoice(FL_OBJECT *, long);
 void doStartupTip(FL_OBJECT *, long);
@@ -49,8 +49,8 @@ void doAccountInfoPaste(FL_OBJECT *, long);
 void doISPDefault(FL_OBJECT *, long);
 void doISPAutoDial(FL_OBJECT *, long);
 void doAutoRedial(FL_OBJECT *, long);
-void doAccountOK(FL_OBJECT *, long);
-void doAccountCancel(FL_OBJECT *, long);
+void doIspPttDropChoice(FL_OBJECT *, long);
+void doIspZoneDropChoice(FL_OBJECT *, long);
 void doAccountTip(FL_OBJECT *, long);
 
 void doRetryInput(FL_OBJECT *, long);
@@ -59,6 +59,7 @@ void doCBYesButton(FL_OBJECT *, long);
 void doCBNoButton(FL_OBJECT *, long);
 void doCBOptions(FL_OBJECT *, long);
 void doCNWaitInput(FL_OBJECT *, long);
+void doLCPWaitInput(FL_OBJECT *, long);
 void doCBInfoTip(FL_OBJECT *, long);
 void doRBYesButton(FL_OBJECT *, long);
 void doRBNoButton(FL_OBJECT *, long);
@@ -68,8 +69,6 @@ void doColumnsInput(FL_OBJECT *, long);
 void doRowsInput(FL_OBJECT *, long);
 void doExpectInput(FL_OBJECT *, long);
 void doSendInput(FL_OBJECT *, long);
-void doDialOK(FL_OBJECT *, long);
-void doDialCancel(FL_OBJECT *, long);
 
 void doCBInfoOK(FL_OBJECT *, long);
 void doCBInfoCancel(FL_OBJECT *, long);
@@ -94,8 +93,6 @@ void doAboutOK(FL_OBJECT *, long);
 void doHelpInfoOK(FL_OBJECT *, long);
 void doStatsInfoOK(FL_OBJECT *, long);
 
-void doCommInfoOK(FL_OBJECT *, long);
-void doCommInfoCancel(FL_OBJECT *, long);
 void doDeviceInput(FL_OBJECT *, long);
 void doModemInitInput(FL_OBJECT *, long);
 void doModemResetInput(FL_OBJECT *, long);
@@ -108,6 +105,7 @@ void doSWCBSDButton(FL_OBJECT *, long);
 void doSWCOffButton(FL_OBJECT *, long);
 void doSWCDeflateButton(FL_OBJECT *, long);
 void doSpeedButton(FL_OBJECT *, long);
+void doCustomSpeedInput(FL_OBJECT *, long);
 void doSWCInput(FL_OBJECT *, long);
 void doAsyncmapInput(FL_OBJECT *, long);
 void doEscapeYesButton(FL_OBJECT *, long);
@@ -123,17 +121,19 @@ void doALNoButton(FL_OBJECT *, long);
 void doRemoteIPInput(FL_OBJECT *, long);
 void doARYesButton(FL_OBJECT *, long);
 void doARNoButton(FL_OBJECT *, long);
-void doTcpipInfoOK(FL_OBJECT *, long);
-void doTcpipInfoCancel(FL_OBJECT *, long);
 void doDRYesButton(FL_OBJECT *, long);
 void doDRNoButton(FL_OBJECT *, long);
-void doRemoteNMInput(FL_OBJECT *, long);
+void doNMInput(FL_OBJECT *, long);
 void doMTUInput(FL_OBJECT *, long);
 void doMRUInput(FL_OBJECT *, long);
-void doUDYesButton(FL_OBJECT *, long);
-void doUDNoButton(FL_OBJECT *, long);
+void doDNSNoButton(FL_OBJECT *, long);
+void doDNSAutoButton(FL_OBJECT *, long);
+void doDNSManualButton(FL_OBJECT *, long);
 void doPDNSInput(FL_OBJECT *, long);
 void doSDNSInput(FL_OBJECT *, long);
+void doDNYesButton(FL_OBJECT *, long);
+void doDNNoButton(FL_OBJECT *, long);
+void doDomainInput(FL_OBJECT *, long);
 void doTcpipInfoTip(FL_OBJECT *, long);
 
 void doOLTimeButton(FL_OBJECT *, long);
@@ -200,28 +200,38 @@ void doZoneNameEditOK(FL_OBJECT *, long);
 void doZoneNameEditCancel(FL_OBJECT *, long);
 
 void doPppdPathInput(FL_OBJECT *, long);
-void doEnvInfoOK(FL_OBJECT *, long);
-void doEnvInfoCancel(FL_OBJECT *, long);
 void doRunPathInput(FL_OBJECT *, long);
+void doLockPathInput(FL_OBJECT *, long);
 void doChatPathInput(FL_OBJECT *, long);
 void doXispUtilsPathInput(FL_OBJECT *, long);
 void doPipePathInput(FL_OBJECT *, long);
 void doPppdDefault(FL_OBJECT *, long);
 void doRunDefault(FL_OBJECT *, long);
+void doLockDefault(FL_OBJECT *, long);
 void doChatDefault(FL_OBJECT *, long);
 void doXispUtilsDefault(FL_OBJECT *, long);
 void doPipeDefault(FL_OBJECT *, long);
 void doEnvInfoTip(FL_OBJECT *, long);
 
+void doOptsFolder(FL_OBJECT *, long);
+void doOptsTabDone(FL_OBJECT *, long);
+void doOptsTabCancel(FL_OBJECT *, long);
+
 /* Forms and Objects */
 
+#define FILE_OPTIONS    1
+#define FILE_HINTS      2
+#define FILE_EXIT       3
+
 #define OPTIONS_ACCOUNT 1
 #define OPTIONS_DIALING 2
 #define OPTIONS_COMM    3
 #define OPTIONS_TCPIP   4
 #define OPTIONS_PATHS   5
+
 #define LOGGING_OPTIONS 1
 #define LOGGING_STATS   2
+
 #define HELP_ABOUT      1
 #define HELP_GENERAL    2
 
@@ -245,7 +255,7 @@ typedef struct {
 	FL_OBJECT *intButton;
 	FL_OBJECT *disButton;
 	FL_OBJECT *quitButton;
-	FL_OBJECT *optMenu;
+	FL_OBJECT *fileMenu;
 	FL_OBJECT *logMenu;
 	FL_OBJECT *hlpMenu;
 	FL_OBJECT *lstBrowser;
@@ -268,8 +278,6 @@ typedef struct {
 	FL_OBJECT *telInput;
 	FL_OBJECT *accInput;
 	FL_OBJECT *pswInput;
-	FL_OBJECT *accOK;
-	FL_OBJECT *accCancel;
 	FL_OBJECT *authType;
 	FL_OBJECT *authPAPButton;
 	FL_OBJECT *authNoneButton;
@@ -277,6 +285,8 @@ typedef struct {
 	FL_OBJECT *authCHAPSButton;
 	FL_OBJECT *UNInput;
 	FL_OBJECT *remoteInput;
+	FL_OBJECT *ispPttDropChoice;
+	FL_OBJECT *ispZoneDropChoice;
 	FL_OBJECT *ISPBrowser;
 	FL_OBJECT *accountInfoAdd;
 	FL_OBJECT *accountInfoDelete;
@@ -308,6 +318,7 @@ typedef struct {
 	FL_OBJECT *rtrInput;
 	FL_OBJECT *dlyInput;
 	FL_OBJECT *CNWaitInput;
+	FL_OBJECT *LCPWaitInput;
 	FL_OBJECT *TTGroup;
 	FL_OBJECT *TTYesButton;
 	FL_OBJECT *TTNoButton;
@@ -324,8 +335,6 @@ typedef struct {
 	FL_OBJECT *ALGroup;
 	FL_OBJECT *expectInput;
 	FL_OBJECT *sendInput;
-	FL_OBJECT *dialOK;
-	FL_OBJECT *dialCancel;
 	FL_OBJECT *tipTimer;
 } FD_dialInfo;
 FD_dialInfo * create_form_dialInfo(void);
@@ -400,8 +409,6 @@ typedef struct {
 	void *vdata;
 	char *cdata;
 	long ldata;
-	FL_OBJECT *commInfoOK;
-	FL_OBJECT *commInfoCancel;
 	FL_OBJECT *modemInitInput;
 	FL_OBJECT *modemResetInput;
 	FL_OBJECT *modemConnectInput;
@@ -416,7 +423,8 @@ typedef struct {
 	FL_OBJECT *SWCOffButton;
 	FL_OBJECT *deviceInput;
 	FL_OBJECT *modemSpeedType;
-	FL_OBJECT *speedButton[8];
+	FL_OBJECT *speedButton[9];
+	FL_OBJECT *customSpeedInput;
 	FL_OBJECT *SWCInput;
 	FL_OBJECT *asyncmapInput;
 	FL_OBJECT *escapeType;
@@ -434,7 +442,7 @@ typedef struct {
 	FL_FORM *tcpipInfo;
 	void *vdata;
 	char *cdata;
-	long ldata;
+	long  ldata;
 	FL_OBJECT *localIPInput;
 	FL_OBJECT *ALTypeGroup;
 	FL_OBJECT *ALYesButton;
@@ -443,19 +451,22 @@ typedef struct {
 	FL_OBJECT *ARTypeGroup;
 	FL_OBJECT *ARYesButton;
 	FL_OBJECT *ARNoButton;
-	FL_OBJECT *tcpipInfoOK;
-	FL_OBJECT *tcpipInfoCancel;
 	FL_OBJECT *DRTypeGroup;
 	FL_OBJECT *DRYesButton;
 	FL_OBJECT *DRNoButton;
-	FL_OBJECT *UDTypeGroup;
-	FL_OBJECT *UDYesButton;
-	FL_OBJECT *UDNoButton;
 	FL_OBJECT *NMInput;
 	FL_OBJECT *MTUInput;
 	FL_OBJECT *MRUInput;
+	FL_OBJECT *NRTypeGroup;
+	FL_OBJECT *DNSNoButton;
+	FL_OBJECT *DNSAutoButton;
+	FL_OBJECT *DNSManualButton;
 	FL_OBJECT *pDNSInput;
 	FL_OBJECT *sDNSInput;
+	FL_OBJECT *DNTypeGroup;
+	FL_OBJECT *DNYesButton;
+	FL_OBJECT *DNNoButton;
+	FL_OBJECT *domainInput;
 	FL_OBJECT *tipTimer;
 } FD_tcpipInfo;
 FD_tcpipInfo * create_form_tcpipInfo(void);
@@ -617,52 +628,133 @@ typedef struct {
 	char *cdata;
 	long  ldata;
 	FL_OBJECT *pppdPathInput;
-	FL_OBJECT *envInfoOK;
-	FL_OBJECT *envInfoCancel;
 	FL_OBJECT *runPathInput;
+	FL_OBJECT *lockPathInput;
 	FL_OBJECT *chatPathInput;
 	FL_OBJECT *xispUtilsPathInput;
 	FL_OBJECT *pipePathInput;
 	FL_OBJECT *pppdDefault;
 	FL_OBJECT *runDefault;
+	FL_OBJECT *lockDefault;
 	FL_OBJECT *chatDefault;
 	FL_OBJECT *xispUtilsDefault;
 	FL_OBJECT *pipeDefault;
 	FL_OBJECT *pppdStatus;
 	FL_OBJECT *runStatus;
+	FL_OBJECT *lockStatus;
 	FL_OBJECT *chatStatus;
 	FL_OBJECT *utilsStatus;
 	FL_OBJECT *tipTimer;
 } FD_envInfo;
 FD_envInfo * create_form_envInfo(void);
 
+typedef struct {
+	FL_FORM *optsTab;
+	void *vdata;
+	char *cdata;
+	long  ldata;
+	FL_OBJECT *optsFolder;
+	FL_OBJECT *accountInfoButton;
+	FL_OBJECT *dialInfoButton;
+	FL_OBJECT *commInfoButton;
+	FL_OBJECT *tcpipInfoButton;
+	FL_OBJECT *envInfoButton;
+	FL_OBJECT *optsTabDone;
+	FL_OBJECT *optsTabCancel;
+} FD_optsTab;
+FD_optsTab * create_form_optsTab(void);
+
 /* Other xISP defines */
 
-#define BU_INTERVAL   2.0	/* check dialer output every 2 seconds */
-#define CT_INTERVAL   5.0	/* update connection time every 5 seconds */
-#define SP_INTERVAL  15.0	/* check link status every 15 seconds */
-#define DIAL_WAIT    30.0	/* max wait seconds for xispdial to open pipe */
-#define UD_WAIT      16.0	/* max wait for ip-up/-down output in pipe */
+#define BU_INTERVAL  2.0			/* check dialer output every 2 seconds */
+#define CT_INTERVAL  5.0			/* update connection time every 5 sec */
+#define SP_INTERVAL  MINSEC_LINK_TO	/* link status check poll inteval */
+#define DIAL_WAIT    30.0			/* max sec for xispdial to open pipe */
+#define UD_WAIT      16.0			/* max sec for ip-up/-down output in pipe */
 #ifdef XPMANIMATE
-#define XPM_INTERVAL  0.3	/* XPM animation trigger interval */
+#define XPM_INTERVAL 0.3			/* XPM animation trigger interval */
 #endif
-#define HINT_TRIGGER  0.2	/* one-shot trigger interval for pop-up hints */
+#define HINT_TRIGGER  0.2	/* one-shot trigger timeout for pop-up hints */
+
+#define LCP_RESTART      3	/* for the pppd cmd line argument with same name */
+#define LCP_RESTART_STR "3"
 
 typedef struct {			/* the color structure */
 	int r, g, b;
 } color_t;
 
-#define BGCOL_DEFAULT {79, 124, 156}	/* default form-background color */
-#define BGCOL_STRING  "#4F7C9C"
+#define BGCOL_DEFAULT  {79, 124, 156}	/* default form-background color */
+#define BGCOL_STRING   "#4F7C9C"
+
+#define PIDFP  "ppp"		/* pppd's PID file prefix */
 
 #define DISCONNECTED 0		/* to specify xISP state */
 #define DIALING      1
 #define CONNECTED    2
 
 #define EMPTY_TIME  "00:00:00"
-#define EMPTY_IP    "---.---.---.---"
-#define EMPTY_SPEED "--,---"
+#define EMPTY_IP    "___.___.___.___"
+#define EMPTY_SPEED "__,___"
 
 #define TIMER_CURSOR_WIDTH 17
 #define TIMER_CURSOR_HEIGHT 19
 
+/* Extra information for input boxes (from XForms internals); this is
+   so that we can display '*' characters instead of ' ' in the secret
+   input password field; there was no other "nicer" way to do it since
+   the API does not give us definitions & control of the SPEC struct */
+
+typedef struct
+{
+    char *str;			/* the input text                  */
+    FL_COLOR textcol;	/* text color                      */
+    FL_COLOR curscol;	/* cursor color                    */
+    int position;		/* cursor position (in chars)      */
+    int beginrange;		/* start of the range              */
+    int endrange;		/* end of the range                */
+    int size;			/* size of the string              */
+    int changed;		/* whether the field has changed   */
+    int how_return;		/* whether to return value always  */
+    int drawtype;		/* if to draw text with background */
+    int noscroll;		/* true if no scrollis allowd      */
+    int maxchars;		/* limit for normal_input          */
+    int attrib1;
+    int attrib2;
+    FL_INPUTVALIDATOR validate;
+
+    /* scroll stuff. */
+    FL_OBJECT *dummy;	/* only for the size of composite */
+    FL_OBJECT *hscroll;
+    FL_OBJECT *vscroll;
+    FL_OBJECT *input;
+    int xoffset;
+    int yoffset;
+    int screenlines;
+    int topline;
+    int lines;			/* total number of lines in the field   */
+    int xpos, ypos;		/* current cursor position in char,line */
+    int cur_pixels;		/* current line length in pixels        */
+    int max_pixels;		/* max length of all lines              */
+    int max_pixels_line;
+    int charh;			/* character height                     */
+    int h, w;			/* text area                            */
+    float hsize, vsize;
+    float hval, vval;
+    float hinc1, hinc2;
+    float vinc1, vinc2;
+    int h_pref, v_pref;	/* scrollbar preference                 */
+    int vw, vw_def;
+    int hh, hh_def;
+    int h_on, v_on;
+    int dead_area, attrib;
+    int no_cursor;
+    int field_char;
+}
+SPEC;
+
+/* exit() replacement for emulating on_exit() functionality */
+
+#ifndef SUNOS41x
+#define exit(x) {extern int exitStatus; exitStatus = x; exit(x);}
+#endif
+
