@@ -1,5 +1,5 @@
 /*
-  Copyright (C) 1997,1998,1999  Dimitrios P. Bouras
+  Copyright (C) 1997-2005  Dimitrios P. Bouras
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -20,7 +20,7 @@
 
 #include <stdio.h>
 #include <stdlib.h>
-#include <varargs.h>
+#include <stdarg.h>
 #include <sys/param.h>
 #include <unistd.h>
 #include <fcntl.h>
@@ -37,11 +37,17 @@
 extern int sys_nerr;
 extern char *sys_errlist[];
 extern int errno;
-extern int fputs(), fscanf(), fprintf(), fclose();
+extern int fputs(), fscanf(), fprintf(), fclose(), sscanf();
 extern char *vsprintf();
 extern void bcopy();
 #endif
 
+#ifdef RUNASEUID
+int xisp_euidaccess();
+#endif
+
+#define MAXLEN_BUF MAXLEN_FNAME
+
 /*+-------------------------------------------------------------------------+
   |                                                                         |
   |                         Global program storage                          |
@@ -49,10 +55,13 @@ extern void bcopy();
   +-------------------------------------------------------------------------+*/
 
 char *scriptfname;						/* login script filename */
+#if !ISPENV_USEVARS
 char *envfname;							/* dialing environment filename */
-char Chat[MAXLEN_FNAME+2];				/* complete path to chat utility */
-char Pipe[MAXLEN_FNAME+2];				/* named pipe report file */
-char Terminal[MAXLEN_FNAME+2];			/* complete path to xispterm */
+#endif /* !ISPENV_USEVARS */
+char Chat[MAXLEN_BUF+2];				/* complete path to chat utility */
+char Pipe[MAXLEN_BUF+2];				/* named pipe report file */
+char Terminal[MAXLEN_BUF+2];			/* complete path to xispterm */
+char envLine[MAXLEN_BUF+2];				/* buf for reading non string vars */
 int pipeFD = 0;							/* report file descriptor */
 int childPID;							/* PID of running child */
 int maxAttempts = MAXNUM_RETRY;			/* max dialing attempts */
@@ -113,7 +122,11 @@ int namedPipe(char *fname)
 	struct stat st;
 	int fd;
 
-	if (access(fname, F_OK) == -1)				/* check to see if it exists */
+#ifdef RUNASEUID
+	if (xisp_euidaccess(fname, F_OK) == -1)		/* check to see if it exists */
+#else
+	if (access(fname, F_OK) == -1)
+#endif
 		doErr("namedPipe: access");				/* nope, this is not right! */
 	else {
 		stat(fname, &st);						/* yes, get the node status */
@@ -138,15 +151,14 @@ int namedPipe(char *fname)
 
 #define MSGLEN_PIPE 256
 
-int pprintf(va_alist) va_dcl
+int pprintf(char *fmt, ...)
 {
-	int iw, bw;
+	unsigned int iw, bw;
 	va_list ap;
-	char *fmt, msg[MSGLEN_PIPE];
+	char msg[MSGLEN_PIPE];
 
-	va_start(ap);								/* start variable arg list */
-	fmt = va_arg(ap, char*);					/* first string is format */
-#ifdef BROKEN_VSPRINTF
+	va_start(ap, fmt);							/* start variable arg list */
+#ifdef BROKEN_SPRINTF
 	vsprintf(msg, fmt, ap);						/* pass rest to vsprintf() */
 	iw = strlen(msg);
 #else
@@ -165,7 +177,7 @@ int pprintf(va_alist) va_dcl
 
 #define MAXARGS_CHILD 16
 
-int child(va_alist) va_dcl
+int child(char *prog, ...)
 {
 	int i, stat;
 	char *args[MAXARGS_CHILD+1];
@@ -182,9 +194,10 @@ int child(va_alist) va_dcl
 	}
 	else {										/* in child process */
 		dup2(pipeFD, 2);						/* tack stderr on pipe input */
-		va_start(ap);							/* start variable arg list */
+		va_start(ap, prog);						/* start variable arg list */
+		args[0] = prog;
 		for (									/* parse program arguments */
-			i=0; i < MAXARGS_CHILD &&			/* to a max of MAXARGS_CHILD */
+			i=1; i < MAXARGS_CHILD &&			/* to a max of MAXARGS_CHILD */
 			(args[i]=va_arg(ap, char*)) !=
 				(char *)0;
 			i++
@@ -215,31 +228,124 @@ void initFnames(void)
 {
 	struct passwd *user;
 
+#ifdef RUNASEUID
+	user = getpwuid(geteuid());
+#else
 	user = getpwuid(getuid());
+#endif
 	scriptfname = (char *)malloc(strlen(user->pw_dir)+1+strlen(SCRIPTFNAME)+1);
+#if !ISPENV_USEVARS
 	envfname = (char *)malloc(strlen(user->pw_dir)+1+strlen(ENVFNAME)+1);
 	if (scriptfname != NULL && envfname != NULL) {
+#else /* ISPENV_USEVARS */
+	if (scriptfname != NULL) {
+#endif /* ISPENV_USEVARS */
 		strcpy(scriptfname, user->pw_dir); strcat(scriptfname, "/");
 		strcat(scriptfname, SCRIPTFNAME);
+#if !ISPENV_USEVARS
 		strcpy(envfname, user->pw_dir); strcat(envfname, "/");
 		strcat(envfname, ENVFNAME);
+#endif /* !ISPENV_USEVARS */
 	}
 	else outOfMem("initFnames");
 }
 
 
-/* getISPenv reads the environment file created by xISP, adjusts
-   xispdial's operation mode flags and and then deletes the file. */
+/* Print message indicating a corrupted environment */
 
 void uEOF(void)
 {
-	pprintf("%s: %s!\n", "xispdial",			/* notify xisp */
-			"environ premature EOF");
-	close(pipeFD);								/* close the report pipe */
-	exit(1);									/* and exit with error */
+	if (pipeFD) {								/* if the pipe to xisp's */
+		pprintf("%s: %s!\n", "xispdial",		/* browser is open, print */
+				"corrupted environment");		/* a message there */
+		close(pipeFD);							/* close the report pipe */
+	}
+	else										/* otherwise print the */
+		fprintf(stderr,							/* message to pppd's script */
+				"%s: %s!\n", "xispdial",		/* error log file */
+				"corrupted environment");
+	exit(1);									/* exit with error */
 }
 
-FILE *envfp;  /* environment file pointer common to readLine(), getISPenv() */
+#if ISPENV_USEVARS
+
+/* Functions for fetching the xispdial environment from an
+   environment variable. */
+
+static const char *eptr = NULL;		/* environment string pointer */
+
+static void init_env()
+{
+	if (eptr == NULL) {
+		eptr = getenv(ENVVAR);
+		if (eptr == NULL)
+			doErr("environment variable " ENVVAR " is not set");
+	}
+}
+
+static char *egets(char *s, int size)
+{
+	register char *t = s;
+
+	init_env();
+	if (eptr == NULL || !*eptr)
+		return NULL;
+	while ((t-s) < (size-1) && *eptr && (*t++ = *eptr++) != '\n');
+	*t = 0;
+	return s;
+}
+
+static int escanf(const char *fmt, void *datap)
+{
+	init_env();
+	if (eptr == NULL || !*eptr)
+		return EOF;
+	egets(envLine, MAXLEN_BUF+1);
+	return sscanf(envLine, fmt, datap);
+}
+
+#define efinish() /* Nothing to do after parsing the environment var */
+
+#else /* !ISPENV_USEVARS */
+
+/* Functions for fetching the xispdial environment from a file.  */
+
+FILE *envfp;  /* environment file pointer */
+
+static void open_envfp()
+{
+	if (envfp == NULL)
+	{
+		envfp = fopen(envfname, "r");		/* open temp environment file */
+		if (envfp == NULL)					/* if not there bail out */
+			doErr("open_envfp: fopen");
+	}
+}
+
+static char *egets(char *s, int size)
+{
+	open_envfp();
+	return fgets(s, size, envfp);
+}
+
+static int escanf(const char *fmt, void *datap)
+{
+	open_envfp();
+	fgets(envLine, MAXLEN_BUF+1, envfp);
+	return sscanf(envLine, fmt, datap);
+}
+
+static void efinish()
+{
+	if (envfp != NULL)
+	{
+		fclose(envfp);
+		envfp = NULL;
+	}
+	unlink(envfname);
+}
+
+#endif /* !ISPENV_USEVARS */
 
 /* The readLine function reads a single line from the environment file
    prepared by xisp, and can be instructed to remove all '\x' escape
@@ -250,12 +356,11 @@ void readLine(char *param, int len, int allowEscape)
 	char *p;
 	int rest;
 
-	if (fgets(param, len+1,						/* read environment line */
-			  envfp) == NULL)
+	if (egets(param, len+1)	== NULL)			/* read environment line */
 		uEOF();									/* bail out on error */
 	param[rest = strlen(param)-1] = 0;			/* kill the ending '\n' */
 	for (p=param; *p;) {						/* scan the line */
-		if (*p == '\\' && !allowEscape) {		/* if '\\' is found and and */
+		if (*p == '\\' && !allowEscape) {		/* if '\\' is found and */
 #ifdef SUNOS41x									/* escape chars not allowed */
 			bcopy(p+1, p, rest);				/* eat-up the '\\' by */
 #else											/* decreasing string len by 1 */
@@ -278,21 +383,21 @@ void readLine(char *param, int len, int allowEscape)
 	}
 }
 
+/* getISPenv() reads the environment variable (or file) created by
+   xISP and adjusts xispdial's operation mode flags; if a file was
+   used for passing the environment, after processing it is deleted */
+
 void getISPenv(void)
 {
 	int ir, i;
 
-	envfp = fopen(envfname, "r");				/* open temp environment file */
-	if (envfp == NULL)							/* if not there bail out */
-		doErr("getISPenv: fopen");
-
-	readLine(Chat, MAXLEN_FNAME, 0);			/* read all needed dialing */
-	readLine(Pipe, MAXLEN_FNAME, 0);			/* environment variables */
-	readLine(Terminal, MAXLEN_FNAME, 0);
-	ir = fscanf(envfp,"%d ",&maxAttempts);
-	ir += fscanf(envfp,"%d ",&sleepDelay);
-	ir += fscanf(envfp,"%d ",&connectWait);
-	ir += fscanf(envfp,"%d ",&numPhones);		/* read # of phone numbers */
+	readLine(Chat, MAXLEN_BUF, 0);				/* read all needed dialing */
+	readLine(Pipe, MAXLEN_BUF, 0);				/* environment variables */
+	readLine(Terminal, MAXLEN_BUF, 0);
+	ir = escanf("%d ",&maxAttempts);
+	ir += escanf("%d ",&sleepDelay);
+	ir += escanf("%d ",&connectWait);
+	ir += escanf("%d ",&numPhones);				/* read # of phone numbers */
 	if (ir < 4)									/* all read so far? */
 		uEOF();									/* nope, bail out */
 	phone = (char **)							/* allocate space for */
@@ -311,13 +416,13 @@ void getISPenv(void)
 		outOfMem("getISPenv");					/* nope, bail out */
 	for (i=0; i<numPhones; i++)
 		readLine(phone[i], MAXLEN_PHONE, 0);	/* read telephone numbers */
-	if (fscanf(envfp,"%X ",&mode) < 1)			/* read connection method */
+	if (escanf("%X ",&mode) < 1)				/* read connection method */
 		uEOF();
 	switch (mode) {
 
 		case SCRIPT_DIALIN:
 			readLine(account,MAXLEN_ACCOUNT,0);	/* read account name and */
-			ir= fscanf(envfp,"%d ",&numSlines);	/* number of script lines */
+			ir= escanf("%d ",&numSlines);		/* number of script lines */
 			if (! ir)							/* bail out on error */
 				uEOF();
 			if (numSlines) {					/* 0 lines is possible */
@@ -347,7 +452,7 @@ void getISPenv(void)
 
 		case SCRIPT_DIALIN|SCRIPT_CALLBACK:
 			readLine(account,MAXLEN_ACCOUNT,0);	/* read account name and */
-			ir= fscanf(envfp,"%d ",&numSlines);	/* number of script lines */
+			ir= escanf("%d ",&numSlines);		/* number of script lines */
 			if (! ir)							/* bail out on error */
 				uEOF();
 			if (numSlines) {					/* 0 lines is possible */
@@ -373,8 +478,8 @@ void getISPenv(void)
 							 4*MAXLEN_SLINE+10+
 							 5*MAXLEN_ACCOUNT, 1);
 			}
-			ir = fscanf(envfp,"%d ",&CBDelay);	/* time wait for call-back */
-			ir+=fscanf(envfp,"%d ",&numCBSlns);	/* number of call-back lines */
+			ir = escanf("%d ",&CBDelay);		/* time wait for call-back */
+			ir+=escanf("%d ",&numCBSlns);		/* number of call-back lines */
 			if (ir < 2)							/* bail out on error */
 				uEOF();
 			if (numCBSlns) {					/* 0 lines is possible */
@@ -404,7 +509,7 @@ void getISPenv(void)
 
 		case SCRIPT_DIALIN|MANUAL_CALLBACK:
 			readLine(account,MAXLEN_ACCOUNT,0);	/* read account name and */
-			ir= fscanf(envfp,"%d ",&numSlines);	/* number of script lines */
+			ir= escanf("%d ",&numSlines);		/* number of script lines */
 			if (! ir)							/* bail out on error */
 				uEOF();
 			if (numSlines) {					/* 0 lines is possible */
@@ -436,7 +541,7 @@ void getISPenv(void)
 					 (sizeof bgcol)-2, 0);
 			readLine(geometry,					/* read window geometry */
 					 MAXLEN_TTPARM, 0);
-			if (fscanf(envfp,"%d ",&CBDelay)<1)	/* time wait for call-back */
+			if (escanf("%d ",&CBDelay)<1)		/* time wait for call-back */
 				uEOF();							/* bail out on error */
 		break;
 
@@ -456,8 +561,8 @@ void getISPenv(void)
 					 (sizeof bgcol)-2, 0);
 			readLine(geometry,					/* read window geometry */
 					 MAXLEN_TTPARM, 0);
-			ir = fscanf(envfp,"%d ",&CBDelay);	/* time wait for call-back */
-			ir+=fscanf(envfp,"%d ",&numCBSlns);	/* number of call-back lines */
+			ir = escanf("%d ",&CBDelay);		/* time wait for call-back */
+			ir+=escanf("%d ",&numCBSlns);		/* number of call-back lines */
 			if (ir < 2)							/* bail out on error */
 				uEOF();
 			if (numCBSlns) {					/* 0 lines is possible */
@@ -492,7 +597,7 @@ void getISPenv(void)
 					 (sizeof bgcol)-2, 0);
 			readLine(geometry,					/* read window geometry */
 					 MAXLEN_TTPARM, 0);
-			if (fscanf(envfp,"%d ",&CBDelay)<1)	/* time wait for call-back */
+			if (escanf("%d ",&CBDelay)<1)		/* time wait for call-back */
 				uEOF();							/* bail out on error */
 		break;
 
@@ -502,7 +607,7 @@ void getISPenv(void)
 
 		case AUTH_DIALIN|AUTH_CALLBACK:
 		case AUTH_DIALIN|NT_RAS_CALLBACK:
-			if (fscanf(envfp,"%d ",&CBDelay)<1)	/* time wait for call-back */
+			if (escanf("%d ",&CBDelay)<1)		/* time wait for call-back */
 				uEOF();							/* bail out on error */
 		break;
 
@@ -525,7 +630,7 @@ void getISPenv(void)
 					 (sizeof bgcol)-2, 0);
 			readLine(geometry,					/* read window geometry */
 					 MAXLEN_TTPARM, 0);
-			if (fscanf(envfp,"%d ",&CBDelay)<1)	/* time wait for call-back */
+			if (escanf("%d ",&CBDelay)<1)		/* time wait for call-back */
 				uEOF();							/* bail out on error */
 		break;
 
@@ -536,8 +641,7 @@ void getISPenv(void)
 	readLine(modemDial, MAXLEN_DIALEXTRA+2, 0);	/* dial char(s) + method */
 	readLine(modemUConn, MAXLEN_MDMSTR, 0);		/* modem connect string */
 
-	fclose(envfp);								/* close and remove the */
-	unlink(envfname);							/* temp environment file */
+	efinish();
 }
 
 
@@ -718,7 +822,7 @@ int callnumber(char *number)
 	fclose(scriptF);							/* close the script file */
 	rcode = child(Chat,"-V","-f",				/* hand it over to chat */
 				  scriptfname,(char *)0);
-	unlink(scriptfname);						/* done with script, delete */
+	unlink(scriptfname);						/* done with script, remove */
 	if (rcode == 0) {							/* chat terminated OK? */
 		if (mode & NT_RAS_CALLBACK)				/* send "done" message */
 			pprintf("\n%s: %s\n","xispdial",	/* according to type */
@@ -761,7 +865,7 @@ int callnumber(char *number)
 				rcode = child(Chat,"-V","-f",	/* hand script over to chat */
 							  scriptfname,
 							  (char *)0);
-				unlink(scriptfname);			/* done with script, delete */
+				unlink(scriptfname);			/* done with script, remove */
 			}
 			if (rcode == 0 &&					/* if we're still OK and */
 				(mode & MANUAL_CALLBACK)) {		/* manual login selected */
@@ -845,7 +949,9 @@ void Term(int signum)
 			if (childPID)						/* if a child is in progress */
 				kill(childPID, SIGTERM);		/* send it SIGTERM also */
 			child(Chat,"-V",""," ",(char *)0);	/* try hanging-up */
-			unlink(envfname);					/* delete environment file */
+#if !ISPENV_USEVARS
+			unlink(envfname);					/* remove environment file */
+#endif /* !ISPENV_USEVARS */
 			unlink(scriptfname);				/* and script file */
 			sleep(MODEM_SLEEP);					/* wait a sec before */
 			init_modem("\\c");					/* reseting the modem */
@@ -885,7 +991,7 @@ int main()
 /* Initialize the modem for dialing
    -------------------------------- */
 	if ((rcode=init_modem(modemInit))) {
-		pprintf("xispdial: chat returned %d!\n", rcode);
+		pprintf("xispdial: chat return status = %d!\n", rcode);
 		pprintf("xispdial: modem init failed!\n");
 		close(pipeFD);
 		return 1;
@@ -897,7 +1003,7 @@ int main()
 		pprintf("\n");
 		callall(attempt);
 		if ((rcode=init_modem(modemInit))) {
-			pprintf("xispdial: chat returned %d!\n", rcode);
+			pprintf("xispdial: chat return status = %d!\n", rcode);
 			pprintf("xispdial: modem re-init failed!\n");
 			close(pipeFD);
 			return 1;
